<?php
/**
 * mapeditapp contains a single class, MapEditApp that controls a map edit
 * application.  The intention is to create a single instance of this object
 * then initialize it and use it to read/write/validate a map file.
 *
 * @author Paul Spencer (spencer@dmsolutions.ca)
 * @project MapEdit
 * @revision $Id: mapeditapp.php,v 1.41.4.1 2005/09/26 14:54:40 zak Exp $
 * @purpose control a mapedit application
 * @copyright Copyright DM Solutions Group Inc 2002
 */

//logger base class
include_once( COMMON."logger/logger.php" );

//objects, elements and factories
include_once( "factory.php" );

//module level variables for XML parsing

//the current object in XML parsing
$szCurObj = "";
//the current element in XML parsing
$szCurType = "";

/**
 * MapEditApp
 */
class MapEditApp extends Logger
{
    /**
     * the base validation object, it will be a DOC object
     */
    var $aValidators;

    /**
     * the object factory
     */
    var $oObjFactory;

    /**
     * the type factory
     */
    var $oTypeFactory;

    /**
     * an instance of MapObj if a mapfile has been parsed
     */
    var $oMap;

    /**
     * instantiate a MapEditApp
     */
    function MapEditApp( )
    {
        $this->Logger( "MapEditApp" );
        $this->logFuncStart( LOG_VERBOSE, "entering MapEditApp()" );

        $this->oObjFactory = new ObjFactory();
        $this->oTypeFactory = new TypeFactory();
        $this->oMap = "";

        //validators are populated from the xml config file
        $this->aValidators = array();

        $this->logFuncEnd( LOG_VERBOSE, "MapEditApp() complete" );
    }

    /**
     * initialize the MapEditApp object on a control file.
     */
    function initialize( $szControlFile)
    {
        $this->logFuncStart( LOG_VERBOSE, "entering initialize" );
        $bResult = true;

        $oXMLParser = xml_parser_create();
        xml_set_element_handler( $oXMLParser, "startElement", "endElement" );
        if ( !($hFile = fopen($szControlFile, "r")) )
        {
            $this->error( 0, "couldn't open XML input $szControlFile" );
            $bResult = false;
        }
        else
        {
            $this->log( LOG_VERBOSE, "starting parse of $szControlFile" );
            $szData = fread( $hFile, filesize( $szControlFile ));
            if ( !xml_parse( $oXMLParser, $szData, true))
            {
                $nXMLErr = xml_get_error_code( $oXMLParser );
                $szXMLErr = xml_error_string( $nXMLErr );
                $nXMLErrLine = xml_get_current_line_number($oXMLParser);
                $szError = "XML Parsing Error $nXMLErr ($szXMLErr) at ";
                $szError .= "line $szXMLErrLine.";
                $this->error( 0, $szError );
                $bResult = false;
            }
            $this->log( LOG_VERBOSE, "parsing complete" );
            //close the file
            fclose( $hFile );
        }

        //free the parser context
        xml_parser_free( $oXMLParser );

        $this->logFuncEnd( LOG_VERBOSE, "done initialize" );
        return $bResult;
    }

    /**
     * parse a map file
     */
    function parseMapFile($szMapFile)
    {
	// Se max execution time more than 30 sec since some mapfile
	// can be very big.
	ini_set("max_execution_time", "60");

        $bResult = true;
        //log entry
        $this->logFuncStart( LOG_VERBOSE, "entering parseMapFile($szMapFile)" );

        //implementation
        if(!is_file($szMapFile))
        {
            $this->error(0,"Invalid Map File ($szMapFile)");
            return false;
        }

        // Pre-parse the mapfile. Transform it to tokens
        //
        //
        //load the php module
        /* -------------------------------------------------------------------- */
        /*      Load required modules                                           */
        /* -------------------------------------------------------------------- */
        if (PHP_OS == "WINNT" || PHP_OS == "WIN32")
        {
            if (!extension_loaded("MapScript"))
            {
                /**
                 * Check if a module name is specified.
                 * If so, use it. If not takes the default
                 * one.
                 */
                if (!isset($_SESSION["gszPHPMapScriptModName"]))
                    $_SESSION["gszPHPMapScriptModName"] = "php_mapscript.dll";

                dl($_SESSION["gszPHPMapScriptModName"]);
            }
        }
        else
        {
            if (!extension_loaded("MapScript"))
            {
                /**
                 * Check if a module name is specified.
                 * If so, use it. If not takes the default
                 * one.
                 */
                if (!isset($_SESSION["gszPHPMapScriptModName"]))
                    $_SESSION["gszPHPMapScriptModName"] = "php_mapscript.so";

                dl($_SESSION["gszPHPMapScriptModName"]);
            }
        }
        $aszMap = ms_TokenizeMap($szMapFile);

        $oTokMapFile = new TokenizedMapFile($aszMap);

        //parse the map file string array
        $this->parseMapFileString($oTokMapFile);

        $this->oMap->saveIndexes();

        //log exit
        $this->logFuncEnd( LOG_ALL, "done parseMapFile" );
        return $bResult;
    }

    /**
     * parse a map string
     */
    function parseMapFileString(&$oTokMapFile)
    {
        $bResult = true;
        //log entry
        $this->logFuncStart( LOG_VERBOSE, "entering parseMapFile" );

        //remove the existing map object if it exists
        if ( isset($this->oMap) )
        unset( $this->oMap );

        //create the map object
        $this->oMap = $this->createObj("MAP");

        //parse the map file string
        $this->oMap->parseMapFileString($oTokMapFile);

        //log exit
        $this->logFuncEnd( LOG_ALL, "done parseMapFile" );
        return $bResult;
    }

    /**
     * create a new map file in memory
     */
    function newMapFile()
    {
        $this->logFuncStart( LOG_VERBOSE, "entering newMapFile" );

        $this->oMap = $this->createObj("MAP");

        $this->logFuncEnd(LOG_ALL, "done newMapFile");
        return true;
    }

    /**
     *
     */
    function saveMapFile( $szName )
    {
        $bResult = true;
        $this->logFuncStart(LOG_VERBOSE, "entering saveMapFile($szName)" );
        $fp = @fopen( $szName, "w+" );
        if($fp === false)
        {
            $this->error( 0, "error saving map to $szName" );
            $bResult = false;
        }
        else
        {
            fwrite( $fp, $this->renderAsMapFileString() );
        }
        $this->oMap->saveIndexes();
        $this->logFuncEnd( LOG_ALL, "done saveMapFile() $bResult" );
        return $bResult;
    }

    /**
     * return a new object from the ObjFactory
     */
    function createObj( $szName )
    {
        $xResult = false;
        //log entry
        $this->logFuncStart( LOG_VERBOSE, "entering createObj($szName)" );

        //implementation
        if(isset($this->aValidators[$szName]))
        $xResult = $this->aValidators[$szName];
        else
        $xResult = $this->oObjFactory->createObj( $szName );

        //log exit
        $this->logFuncEnd( LOG_ALL, "done createObj($szName)" );
        return $xResult;
    }

    /**
     * return a new type from the TypeFactory
     */
    function createType( $szName )
    {
        $mResult = false;
        //log entry
        $this->logFuncStart( LOG_VERBOSE, "entering createType($szName)" );

        //implementation
        $mResult = $this->oTypeFactory->createType( $szName );

        //log exit
        $this->logFuncEnd( LOG_ALL, "done createType($szName)" );
        return $mResult;
    }

    function &getObjByIndex( $szIndex )
    {
        if (is_object($this->oMap))
        {
            $oObj = &$this->oMap;
            $oObj = &$oObj->getObjByIndex( $szIndex );
            return $oObj;
        }
        else
        {
            $this->log( LOG_QUIET, "map object not initialized in getObjByIndex()" );
            return false;
        }
    }

    function addNewObject( $szIndex, $szName )
    {
        $this->log( LOG_ALL, "addNewObject $szName on index $szIndex called" );
        if (is_object($this->oMap) && $szName != "MAP")
        {
            $szIndex = $this->oMap->addNewObject( $szIndex, $szName );
            $this->log( LOG_QUIET, "addNewObject returning index $szIndex" );
            return $szIndex;
        }
        else
        {
            $this->error( 0, "error creating $szName" );
            return false;
        }
    }

    function processFormVars( $szIndex, $axFormVars )
    {
        $this->log( LOG_QUIET, "processFormVars on $szIndex" );
        return $this->oMap->processFormVars( $szIndex, $axFormVars );
    }
    function renderAsMapFileString()
    {
        $szResult = "";
        $this->logFuncStart( LOG_VERBOSE, "entering renderAsMapFileString()" );

        if(is_object($this->oMap))
        $szResult = $this->oMap->renderAsMapFileString();
        else
        {
            $this->error(0, "tried to render without a valid map object");
        }
        $this->logFuncEnd( LOG_ALL, "ending renderAsMapFileString()");
        return $szResult;
    }

    function renderAsTreemenuString( )
    {
        $szResult = "";
        $this->logFuncStart( LOG_VERBOSE, "entering renderAsTreemenuString()" );

        if(is_object($this->oMap))
        $szResult = $this->oMap->renderAsTreemenuString( "." );
        else
        $this->error(0, "tried to render without a valid map object");

        $this->logFuncEnd( LOG_ALL, "ending renderAsTreemenuString()");
        return $szResult;
    }
}

/**
 * startElement is the callback function used by the xml parser to
 * indicate that it has found a new start tag in the xml.  Logic is
 * to create an object from the ObjectFactory for each start tag and
 * track the current object's  name globally.  If the current object is
 * valid then subsequent start tags are created as elements and are added
 * to the current object.  If the current element is valid and a new start
 * tag is encountered, then the current element must be a complex element
 * and the new element is added to the current element (which is already
 * part of the object).  endElement handles closing tags.
 *
 * @param parser - the xml parsing context object for use in xml_xxx
 *        functions
 * @param szName - the name of the current tag
 * @param aAttrs - an associative array of attribute name => value pairs
 *
 * @return this function doesn't return a value
 *
 * @see endElement
 */
function startElement( $parser, $szName, $aAttrs )
{
    global $gApp, $gLogFile, $szCurObj, $szCurType;
    //log entry

    $gLogFile->logMessage( LOG_VERBOSE, "starting tag $szName" );
    $gLogFile->logMessage( LOG_ALL, "curType = $szCurType, curObj = $szCurObj" );
    $gLogFile->indent();

    if ( !isset( $gApp ) or !is_object( $gApp ) )
    {
        $_SESSION["gErrorManager"]->setError( 0,
                                  "application object invalid in XML parser");
        return;
    }

    //first look for a valid object - make one if it doesn't exist
    if ( $szCurObj == "" )
    {
        $gLogFile->logMessage( LOG_ALL, "starting new object for $szName" );
        //create a new object
        $oObj = $gApp->createObj( $szName );
        //set it's attributes
        if ( isset($oObj) && is_object( $oObj ) )
        {
            $gLogFile->logMessage( LOG_ALL,
                                   "Created object ".$oObj->mszName );
            if ( isset( $aAttrs['NAME']))
                $oObj->mszTitle = $aAttrs['NAME'];
            if ( isset( $aAttrs['SUBOBJECTS']))
                $oObj->maszValidObjs = explode( "|", $aAttrs['SUBOBJECTS']);
            if ( isset( $aAttrs['MULTIPLICITY']))
                $oObj->mszMultiplicity = $aAttrs['MULTIPLICITY'];
            if ( isset( $aAttrs['RESTRICTED']))
                $oObj->maszRestricted = explode( "|", $aAttrs['RESTRICTED']);

            $szCurObj = $oObj->mszName;
            $gApp->aValidators[$szName] = $oObj;
        }
    }
    else
    {
        //otherwise get it from the gApp object and see
        if ( !isset( $gApp->aValidators[$szCurObj]))
        {
            $_SESSION["gErrorManager"]->setError( 0, "invalid object ($szCurObj)");
            return;
        }

        $oObj = &$gApp->aValidators[$szCurObj];
        //create a new Type object for the current type element
        if ( $szCurType == "" )
        {
            $gLogFile->logMessage( LOG_ALL, "starting new Type for $szName" );

            if ( !isset( $aAttrs['TYPE']))
            {
                $_SESSION["gErrorManager"]->setError( 0,
                                          "invalid Type Element - $szName - no TYPE attribute");
                return;
            }
            $oType = $gApp->createType( $aAttrs['TYPE']);

            if ( !is_object( $oType ))
            {
                $_SESSION["gErrorManager"]->setError( 0, "invalid TYPE on line ".
                                          xml_get_current_line_number($parser));
                return;
            }
            $oType->setName( $szName );
            $oType->mszTitle = $aAttrs['NAME'];
            //this is a special case for enumeration types
            if( isset($aAttrs['VALUE'] ) )
                $oType->setEnumValues( $aAttrs['VALUE'] );
            if( isset( $aAttrs['DEFAULT'] ))
                if ( isset($aAttrs['VALUE']) && $aAttrs['VALUE'] == "")
                    $oType->setEnumValues( $aAttrs['DEFAULT'] );
                else
                    $oType->setValue( $aAttrs['DEFAULT'] );
            if( isset( $aAttrs['VALUETYPE'] ) )
                $oType->setValueType($aAttrs['VALUETYPE']);
            if( isset( $aAttrs['APPLIESTO'] ) )
                $oType->setAppliesTo(explode("|", $aAttrs['APPLIESTO']));

            $oObj->addType( $oType );
            $szCurType = $oType->mszName;
        }
        else
        {
            $gLogFile->logMessage( LOG_ALL, "adding to complextype ".
                                   "$szCurType for $szName" );

            //an existing element, must be of type COMPLEXTYPE
            if ( !isset( $oObj->maoTypes[$szCurType]))
            {
                $_SESSION["gErrorManager"]->setError( 0, "invalid type ($szCurType) on ".
                                          "line ".xml_get_current_line_number($parser) );
                return;
            }
            $oType = &$oObj->maoTypes[ $szCurType ];
            if ( $oType->mszType == "COMPLEXTYPE" )
            {
                //complex types are special cases
                $oElement = $gApp->createType( $aAttrs['TYPE'] );
                $oElement->setName( $szName );
                $oElement->mszTitle = $aAttrs['NAME'];
                $oType->addType( $oElement );
            }
            else
            {
                $szContType = $oType->mszType;
                $szContName = $oType->mszName;
                $szMessage  = "invalid type nesting, tried to put a ".
                $aAttrs['TYPE']." into a ".$szContType.
                "on line ".xml_get_current_line_number( $parser );
                $_SESSION["gErrorManager"]->setError( 0, $szMessage);
            }

        }
    }
}

/**
 * handle the closing tag of an XML element.  If there is an element
 * open, set it to empty, otherwise if there is an object open, add it
 * to the MapEditApp validator array
 */
function endElement( $parser, $szName )
{
    global $gApp, $gLogFile, $szCurObj, $szCurType;

    $gLogFile->logMessage( LOG_VERBOSE, "endElement called for $szName" );
    $gLogFile->logMessage( LOG_VERBOSE, "curType = $szCurType, curObj = $szCurObj" );

    if ( $szCurType == $szName )
    {
        $gLogFile->logMessage( LOG_VERBOSE, "finishing element ".$szCurType );
        $szCurType = "";
    }
    elseif ( $szCurObj == $szName )
    {
        $gLogFile->logMessage( LOG_VERBOSE, "finishing object ".$szName );
        $szCurObj = "";
    }

    //log exit
    $gLogFile->dedent();
    $gLogFile->logMessage( LOG_VERBOSE, "finished tag for $szName");
}
?>
